//
// JUMA-TX136 Transmitter Controller
// F4GCB 10.2014
//
// Microchip MPLAB + C30 compiler
// MCU dsPIC30F6014A
//
// Generates a WSPR message from call locator and power to a symbol/tone
//
// Acknowledgement :
// The WSPR message algorithm is derived from Fortran and C files found the K1JT WSPR source code. 
// Portions of the WSPR message algorithm is the work of Andy Talbot, G4JNT
// Portions of this code is derived from the work of Gene Marcus, W3PM

#include "juma-tx500.h"							// JUMA-TRX2 hardware specific definitions
#include "tx500_eeprom.h"						// EEPROM storage structure
#include "lcd-tx500.h"							// LCD specific definitions

extern union
	{
	struct defval defval;
	unsigned int storage[sizeof(struct defval)/2];
	}eeprom;

extern union
	{
	struct calval calval;
	unsigned int ee[sizeof(struct calval)/2];
	}cal;

// 1ms interupt routine variables and functions
extern long busy_counter;						// tone generator busy(ms), busy if != 0
extern long cw_break_time;						// set value for the timer
extern long cw_break_timer;						// cw break timer
extern long cw_period;							// cw impulse period for the keyer code(ms)
extern int seconds;								// running 1s timer for WSPR
extern int minutes;								// running 1min timer for WSPR

extern char lcd_m_buf[10];						// LCD morse code display buffer
extern const int wspr_pwr_list[19];				// WSPR possible power dBm

char call[7];
char grid[5];
int ndbm;

unsigned long code1;			// callsign compressed
unsigned long code2;			// grid and power level compressed
unsigned short int code[11];	// 50 bits callsign grid and power level compressed
int symbol_conv[162];			// symbols after convolutional encoding
int symbol[162];				// final symbols after interleaving
int timeslots ;					// number of time slots passed
int start_ts;					// start timeslot available

extern void play_wspr_sym(int shift);

// 162 bit synchronization vector
const int sync_vec[162] = {
	1,1,0,0,0,0,0,0,1,0,0,0,1,1,1,0,0,0,1,0,0,1,0,1,1,1,1,0,0,0,0,0,0,0,1,0,0,1,0,1,0,0,0,0,0,0,1,0,
  	1,1,0,0,1,1,0,1,0,0,0,1,1,0,1,0,0,0,0,1,1,0,1,0,1,0,1,0,1,0,0,1,0,0,1,0,1,1,0,0,0,1,1,0,1,0,1,0,
  	0,0,1,0,0,0,0,0,1,0,0,1,0,0,1,1,1,0,1,1,0,0,1,1,0,1,0,0,0,1,1,1,0,0,0,0,0,1,0,1,0,0,1,1,0,0,0,0,
  	0,0,0,1,1,0,1,0,1,1,0,0,0,1,1,0,0,0
	};

int wspr_sym_idx;				// currently playing WSPR symbol list index
int wspr_sym_playing;			// symbol playing, keyer reserved, 1=playing

// normalize characters 0..9 A..Z space in order 0..36
char chr_normf(char bc ) 
{
	char cc = 36;
  	if (bc >= '0' && bc <= '9') cc = bc - '0';
  	if (bc >= 'A' && bc <= 'Z') cc = bc - 'A' + 10;
  	if (bc >= 'a' && bc <= 'z') cc = bc - 'a' + 10;  
  	if (bc == ' ' ) cc = 36;

  	return(cc);
}

// coding of callsign
void code_call(void)
{
	int i;

	// the third character must always be a number
  	if (chr_normf(call[2]) > 9) 
  	{
		for (i=5; i>0; i--) 
    		call[i] = call[i-1];
    	call[0] = ' ';
  	}

  	code1 = chr_normf(call[0]);
  	code1 = code1 * 36 + chr_normf(call[1]);
  	code1 = code1 * 10 + chr_normf(call[2]);
  	code1 = code1 * 27 + chr_normf(call[3])-10;
  	code1 = code1 * 27 + chr_normf(call[4])-10;
  	code1 = code1 * 27 + chr_normf(call[5])-10;
}

// coding of grid and power level
void code_grid_power(void)
{
  	// coding of grid
  	code2 = 179 - 10 * (chr_normf(grid[0]) - 10) - chr_normf(grid[2]);
  	code2 = code2 * 180 + 10 * (chr_normf(grid[1]) - 10) + chr_normf(grid[3]);

	// add coding of power level
	code2 = code2 * 128 + ndbm + 64;
}

// code1 and code2 truncated and combined into 50 bits code
void code_50_bits(void)
{
	int i;
	unsigned long temp;

  	// merge coded callsign into message array code[]
  	temp = code1;
	code[0]= (temp >> 20) & 0xFF;
  	temp = code1;
	code[1]= (temp >> 12) & 0xFF;
  	temp = code1;
	code[2]= (temp >> 4) & 0xFF;
  	temp = code1;
	code[3]= (temp << 4) & 0xFF;

	// merge coded grid and power level into message array code[]
  	temp = code2;
	code[3]= code[3] + ((0x0F & temp >> 18) & 0xFF);
  	temp = code2;
	code[4]= (temp >> 10) & 0xFF;
  	temp = code2;
	code[5]= (temp >> 2) & 0xFF;
  	temp = code2;
	code[6]= (temp << 6) & 0xFF;

	// the lowest 6 bytes are set to 0
	for (i=7; i<12; i++)
		code[i] = 0;
}

//calculate the single bit parity (XOR)
int parity(unsigned long value)
{
  	int bit_to_1 = 0;
  	while(value != 0)
  	{
    	bit_to_1++;
    	value &= (value - 1);
  	}

  	return (bit_to_1 & 1);
}

// convolutional encoding of message array code[] into a 162 bit stream
void encode_conv(void)
{
  	int byte_cnt = 0;
  	int cnt = 0;
  	int i, byte_code;
  	unsigned long shift_register = 0;

  	byte_code = code[0];
	for (i=0; i<81 ;i++)
	{
		// the 81 bits (including the 31 trailing zeros) are read out MSB first
		if (i % 8 == 0 )
		{
      		byte_code = code[byte_cnt];
      		byte_cnt++;
    	}
    	if (byte_code & 0x80) shift_register = shift_register | 1;

    	symbol_conv[cnt++] = parity(shift_register & 0xF2D05351);
    	symbol_conv[cnt++] = parity(shift_register & 0xE4613C47);

    	byte_code = byte_code << 1;
    	shift_register = shift_register << 1;
  	}
}

// interleave reorder the 162 data bits and and merge table with the sync vector
void interleave_sync(void)
{
  	int address, address_rev ,bit_pos, bit_test;
  	int cnt = 0;

  	for (address=0; address<=255; address++)
	{
    	bit_test = 1;
    	address_rev = 0;
		// bit address reverse eg address = 1 give address_rev = 128
    	for (bit_pos=0; bit_pos<8 ; bit_pos++)
		{
      		if (address & bit_test) address_rev = address_rev | (0x80 >> bit_pos);
      		bit_test = bit_test << 1;
    	}
		// interleaving process
    	if (address_rev < 162 )
		{
      		symbol[address_rev] = sync_vec[address_rev] + 2 * symbol_conv[cnt];
      		cnt++;
    	}
  	}
}

void disp_wspr_code(void)
{
	int x;

	x = wspr_sym_idx;
	lcd_m_buf[0] = 'S';
	lcd_m_buf[1] = ':';
	lcd_m_buf[2] = ((x/100)+'0');
	x-=(x/100)*100;
	lcd_m_buf[3] = ((x/10)+'0');
	x-=(x/10)*10;
	lcd_m_buf[4] = ((x/1)+'0');
	lcd_m_buf[5] = ' ';
	x = symbol[wspr_sym_idx - 1];
	lcd_m_buf[6] = ((x/1)+'0');
	lcd_m_buf[7] = ' ';
}

// Standard dBm WSPR (types 0 3 7 10 13 17 ... 60)
int wspr_ndbm(int dbm)
{
	int nu[10] = {0,-1,1,0,-1,2,1,0,-1,1};
	int ndbm_std;

	ndbm_std = dbm;
	if (ndbm_std < 0)
		ndbm_std = 0;
	if (ndbm_std > 60)
		ndbm_std = 60;
	return ndbm_std + nu[ndbm_std % 10];
}

// decrease dBm level in function TX power
int ndbm_rfpwr(void)
{
	int nu_rfpwr[4] = {13,6,3,0};

	return wspr_ndbm(eeprom.defval.wspr_ndbm - nu_rfpwr[eeprom.defval.rfpwr]);
}

// init WSPR beacon
void init_wspr_beacon(void)
{
	int i;

	i = 0;
	while(eeprom.defval.wspr_call[i]!= 0)
	{
		call[i] = eeprom.defval.wspr_call[i];
		i++;
	}
	i = 0;
	while(eeprom.defval.wspr_grid[i]!= 0)
	{
		grid[i] = eeprom.defval.wspr_grid[i];
		i++;
	}
	ndbm = ndbm_rfpwr();
	
	code_call();
  	code_grid_power();
  	code_50_bits();
  	encode_conv();
  	interleave_sync();
	timeslots = 0;
	wspr_sym_playing = 0;
	start_ts = 0;
}

// this function is called continously from main when WSPR beacon is on
// init beacon must be done before
void run_wspr_beacon(void)
{
	if(wspr_sym_playing)							// wait for machine free
		return;
	
	if (eeprom.defval.beacon_tx_on == 0)			// was last round, do stop procedures
	{
		cw_break_timer = 10;						// fast release 10ms
		cw_break_time = cal.calval.cw_break_time;	// normal cw break time
		return;
	}

	if (eeprom.defval.mode == MODE_WSPR2 && !(minutes % 2) && seconds == 1)			// timeslot every 2 minutes and 1 second
		start_ts++;
	else if (eeprom.defval.mode == MODE_WSPR15 && !(minutes % 15) && seconds == 1)	// timeslot every 15 minutes and 1 second
		start_ts++;
	else start_ts = 0;

	// enable WSPR TX for the timeslot then in function of the frame
	if (start_ts == 1)
	{ 
		if (eeprom.defval.wspr_frame == 0)
		{
			wspr_sym_playing = 1;
			wspr_sym_idx = 0;
		}
		else if (!(timeslots % eeprom.defval.wspr_frame))
		{
			if (timeslots == 12)					// reset timeslots counter to avoid overflow
				timeslots = 0;
			wspr_sym_playing = 1;
			wspr_sym_idx = 0;
			timeslots++;
		}
		else
			timeslots++;
	}
}

// Beacon WSPR generator
// 1ms IRQ function, feed beacon WSPR symbols info to the keyer code
void wspr_sym_irq(void)
{
	if(wspr_sym_playing && busy_counter == 0)	// run logic only when play symbol active and keyer free
	{
		if (wspr_sym_idx == 162)				// end of timeslot
		{
			wspr_sym_idx = 0;
			wspr_sym_playing = 0;
			if (eeprom.defval.beacon_tx_on > 0)	// not continous(-1) or rounds left(positive)
				eeprom.defval.beacon_tx_on--;
		}
		else
		{
			play_wspr_sym(symbol[wspr_sym_idx]);
			wspr_sym_idx++;
			disp_wspr_code();
		}
	}
	else if (!wspr_sym_playing && lcd_m_buf[2] != '-')
		sprintf(lcd_m_buf, "S:--- -  ");
}

// needed because linker script puts .const in bootloader memory area
// if the .text length insn't enough long !
// no elegant but it works waiting a better solution
// threshold effect with .text length ?
void program_memory_busy(void)
{
	int i;

	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
	i = 0;
}

